@@ -10,7 +10,7 @@ use Carp ();
 use Scalar::Util ();
 
 use Text::Xslate::PP::Const;
-use Text::Xslate::Util qw(p escaped_string html_escape);
+use Text::Xslate::Util qw(p mark_raw unmark_raw html_escape);
 
 use constant TXframe_NAME       => Text::Xslate::PP::TXframe_NAME;
 use constant TXframe_OUTPUT     => Text::Xslate::PP::TXframe_OUTPUT;
@@ -152,7 +152,7 @@ sub op_fetch_field_s {
 sub op_print {
     my $sv = $_[0]->{sa};
 
-    if ( ref( $sv ) eq 'Text::Xslate::EscapedString' ) {
+    if ( ref( $sv ) eq 'Text::Xslate::Type::Raw' ) {
         if(defined ${$sv}) {
             $_[0]->{ output } .= ${$sv};
         }
@@ -357,12 +357,17 @@ sub op_max_index {
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_builtin_raw {
-    $_[0]->{sa} = escaped_string($_[0]->{sa});
+sub op_builtin_mark_raw {
+    $_[0]->{sa} = mark_raw($_[0]->{sa});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_builtin_html {
+sub op_builtin_unmark_raw {
+    $_[0]->{sa} = unmark_raw($_[0]->{sa});
+    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
+}
+
+sub op_builtin_html_escape{
     $_[0]->{sa} = html_escape($_[0]->{sa});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
@@ -413,14 +418,14 @@ sub op_ge {
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_function {
+sub op_symbol {
     my $name = $_[0]->op_arg;
 
-    if ( my $func = $_[0]->function->{ $name } ) {
+    if ( my $func = $_[0]->symbol->{ $name } ) {
         $_[0]->{sa} = $func;
     }
     else {
-        Carp::croak( sprintf( "Oops: Undefined function %s", $name ) );
+        Carp::croak( sprintf( "Undefined symbol %s", $name ) );
     }
 
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
@@ -478,7 +483,7 @@ sub op_macro_end {
         $st->{targ} = $st->{ output };
     }
     else {
-        $st->{targ} = escaped_string( $st->{ output } );
+        $st->{targ} = mark_raw( $st->{ output } );
     }
 
     $st->{sa} = $st->{targ};
@@ -674,11 +679,13 @@ __END__
 
 =head1 NAME
 
-Text::Xslate::PP::Opcode - Text::Xslate opcodes in pure Perl
+Text::Xslate::PP::Opcode - Text::Xslate opcode implementation in pure Perl
 
 =head1 DESCRIPTION
 
-This module is used by Text::Xslate::PP internally.
+This module is a pure Perl implementation of the Xslate opcodes.
+
+The is enabled with C<< $ENV{ENV}='pp=opcode' >>.
 
 =head1 SEE ALSO
 
