@@ -29,9 +29,9 @@
 
 /* xslate stuff */
 
-#define TX_ESC_CLASS   "Text::Xslate::EscapedString"
-#define TX_MACRO_CLASS "Text::Xslate::Macro"
+#define TX_RAW_CLASS   "Text::Xslate::Type::Raw"
 #define TX_PAIR_CLASS  "Text::Xslate::Type::Pair"
+#define TX_MACRO_CLASS "Text::Xslate::Macro"
 
 /* arbitrary initial buffer size */
 #define TX_HINT_SIZE 128
@@ -129,7 +129,7 @@ struct tx_state_s {
     I32 current_frame; /* current frame index */
     SV** pad;          /* AvARRAY(frame[current_frame]) + 3 */
 
-    HV* function; /* name => \&body | [macro object] */
+    HV* symbol; /* symbol table (e.g. name => \&body | [macro object]) */
 
     U32 hint_size; /* suggested template size (bytes) */
 
